import copy
import enum
import typing as t

import pydantic

import openapydantic

Field = pydantic.Field

OpenApiBaseModel = openapydantic.common.OpenApiBaseModel
ComponentType = openapydantic.common.ComponentType
HTTPStatusCode = openapydantic.common.HTTPStatusCode
MediaType = openapydantic.common.MediaType


class ComponentsResolver:
    with_ref: t.Dict[str, t.Any] = {}
    without_ref: t.Dict[str, t.Any] = {}
    ref_find = False
    consolidate_count = 0
    find_ref_count = 0

    @classmethod
    def init(cls):
        cls.with_ref = {}
        cls.without_ref = {}
        cls.ref_find = False

        for elt in ComponentType:
            cls.with_ref[elt.name] = {}
            cls.without_ref[elt.name] = {}

    @staticmethod
    def _validate_ref(ref: str) -> None:
        if ".yaml" in ref or ".json" in ref:
            raise NotImplementedError("File reference not implemented")
        if not ref.startswith("#/"):
            raise ValueError(f"reference {ref} has invalid format")

    @classmethod
    def _find_ref(
        cls,
        *,
        obj: t.Any,
    ):
        cls.find_ref_count = cls.find_ref_count + 1
        if cls._find_ref:
            return

        if isinstance(obj, list):
            for elt in obj:  # type: ignore
                cls._find_ref(obj=elt)

        if isinstance(obj, dict):
            ref = obj.get("$ref")  # type: ignore
            if ref:
                cls._validate_ref(ref=ref)  # type: ignore
                cls.ref_find = True

            for key in obj:  # type: ignore
                cls._find_ref(obj=obj.get(key))  # type: ignore

    @classmethod
    def _search_component_for_ref(
        cls,
        *,
        key: str,
        value: t.Dict[str, t.Any],
        component_type: ComponentType,
    ):
        cls.ref_find = False
        cls._find_ref(
            obj=value,
        )

        if cls.ref_find:
            cls.with_ref[component_type.name][key] = value
        else:
            cls.without_ref[component_type.name][key] = value

    @classmethod
    def _search_components_for_ref(
        cls,
        *,
        components: t.Dict[str, t.Any],
        component_type: ComponentType,
    ):
        for key, value in components.items():
            cls._search_component_for_ref(
                key=key,
                value=value,
                component_type=component_type,
            )

    @staticmethod
    def _get_component_object(
        component_type: ComponentType,
        values: t.Dict[str, t.Any],
    ) -> t.Dict[str, t.Any]:
        if component_type == ComponentType.schemas:
            component = Schema(**values)
        elif component_type == ComponentType.headers:
            component = Header(**values)
        elif component_type == ComponentType.responses:
            component = Response(**values)
        elif component_type == ComponentType.parameters:
            component = Parameter(**values)
        elif component_type == ComponentType.examples:
            component = Example(**values)
        elif component_type == ComponentType.request_bodies:
            component = RequestBody(**values)
        elif component_type == ComponentType.links:
            component = Link(**values)
        elif component_type == ComponentType.callbacks:
            component = PathItem(**values)

        return component.as_clean_dict()

    @classmethod
    def _consolidate_components(
        cls,
        component_type: ComponentType,
    ) -> None:
        cls.consolidate_count = cls.consolidate_count + 1
        component_dict = {}
        with_ref_copy = copy.deepcopy(cls.with_ref[component_type.name])
        # print(f"INFO NEW CONSOLIDATE")
        # print(f"with_ref remaining:{len(with_ref_copy)}")
        # print(f"remaining:{[k for k in with_ref_copy]}")
        for key, values in with_ref_copy.items():
            cls.ref_find = False
            # print(f"INFO:{key}:{values}")

            component_dict = ComponentsResolver._get_component_object(
                component_type=component_type,
                values=values,
            )
            cls._search_component_for_ref(
                key=key,
                value=component_dict,  # type: ignore
                component_type=component_type,
            )
            # print(cls.ref_find)
            # print(key)

            if cls.ref_find:
                # print(f"BAD INFO:still ref in:{key}:{component_dict}")
                cls.with_ref[component_type.name][key] = component_dict
            else:
                # print(f"GOOD INFO:adding without ref:{key}:{component_dict}")
                cls.without_ref[component_type.name][key] = component_dict
                del cls.with_ref[component_type.name][key]

        if len(cls.with_ref[component_type.name]):
            cls._consolidate_components(component_type=component_type)

    @classmethod
    def resolve(
        cls,
        *,
        raw_api: t.Dict[str, t.Any],
    ):
        cls.init()

        components = raw_api.get("components")
        if not components:
            print("No components in this api")
            return

        for elt in ComponentType:
            component = components.get(elt.value)
            if component:
                cls._search_components_for_ref(
                    components=component,
                    component_type=elt,
                )

        for elt in ComponentType:
            component = components.get(elt.value)
            if component:
                cls._consolidate_components(component_type=elt)


class RefModel(OpenApiBaseModel):
    ref: t.Optional[str] = Field(
        None,
        alias="$ref",
    )

    @staticmethod
    def _get_ref_data(ref: str) -> t.Tuple[ComponentType, str]:
        ref_split = ref.split("/")  # format #/components/schemas/Pet
        ref_type = ComponentType(ref_split[2])
        ref_key = ref_split[-1]
        return ref_type, ref_key

    @pydantic.root_validator(
        pre=True,
        allow_reuse=True,
    )
    def validate_root(
        cls,
        values: t.Dict[str, str],
    ) -> t.Dict[str, t.Any]:
        ref = values.get("$ref")
        # print("====== VALIDATE ROOT ======")
        # print(f"ref:{ref}")
        if ref:
            # load reference from ComponentsResolver
            ref_type, ref_key = cls._get_ref_data(ref)
            # print(f"ref_type:{ref_type.name}")
            # print(f"ref_key:{ref_key}")
            ref_found: t.Dict[str, t.Any] = ComponentsResolver.without_ref[
                ref_type.name
            ].get(ref_key)

            # print(f"ref_found:{ref_found}")
            if not ref_found:
                raise ValueError(f"Reference not found:{ref_type}/{ref_key}")

            return ref_found
        return values

    #     # check spec extension:
    #     native_attr = set(cls.__fields__.keys())  # all class attr key
    #     setted_attr = [k for k, v in values.items() if v]  # setted attr key
    #     extra_attr = [k for k in setted_attr if k not in native_attr]  # difference
    #     clean_extra_attr = list(filter(lambda x: (x != "$ref"), extra_attr))
    #     for attr in clean_extra_attr:
    #         if not attr.startswith("x-"):
    #             raise ValueError(
    #                 f"Schema extension:{attr} must be conform to openapi "
    #                 f"specication (^x-)"
    #             )
    #     return values


class BaseModelForbid(RefModel):
    class Config:
        extra = "forbid"


class BaseModelAllow(RefModel):
    class Config:
        extra = "allow"


class JsonType(enum.Enum):
    integer = "integer"
    string = "string"
    array = "array"
    object_ = "object"
    boolean = "boolean"
    number = "number"


class SecurityIn(enum.Enum):
    query = "query"
    header = "header"
    cookie = "cookie"


Discriminator = t.Mapping[str, t.Dict[str, str]]


class XML(BaseModelForbid):
    name: t.Optional[str]
    namespace: t.Optional[pydantic.AnyUrl]
    prefix: t.Optional[str]
    attribute: t.Optional[bool]
    wrapped: t.Optional[bool]


class ExternalDocs(BaseModelForbid):
    description: t.Optional[str]
    url: t.Optional[pydantic.AnyUrl]


SchemaUnion = t.Union[
    t.Mapping[str, "Schema"],
    "Schema",
]


class Schema(BaseModelAllow):
    title: t.Optional[str]
    multiple_of: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="multipleOf",
    )
    maximum: t.Optional[int]
    exclusive_maximum: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="exclusiveMaximum",
    )
    minimum: t.Optional[int]
    exclusive_minimum: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="exclusiveMinimum",
    )
    max_length: t.Optional[int] = Field(
        None,
        alias="maxLength",
    )
    min_length: t.Optional[int] = Field(
        None,
        alias="minLength",
    )
    pattern: t.Optional[t.Pattern]
    max_items: t.Optional[int] = Field(
        None,
        alias="maxItems",
    )
    min_items: t.Optional[int] = Field(
        None,
        alias="minItems",
    )
    unique_items: t.Optional[t.Any] = Field(
        None,
        alias="uniqueItems",
    )
    max_properties: t.Optional[int] = Field(
        None,
        alias="maxProperties",
    )
    min_properties: t.Optional[int] = Field(
        None,
        alias="minProperties",
    )
    required: t.Optional[t.List[str]]
    enum: t.Optional[t.List[str]]
    type: t.Optional[JsonType]
    all_of: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="allOf",
    )
    one_of: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="oneOf",
    )
    any_of: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="anyOf",
    )
    not_: t.Optional[t.List[SchemaUnion]] = Field(
        None,
        alias="not",
    )
    items: t.Optional[SchemaUnion]
    properties: t.Optional[SchemaUnion]
    additional_properties: t.Optional[SchemaUnion] = Field(
        None,
        alias="additionalProperties",
    )
    description: t.Optional[str]
    format: t.Optional[str]
    default: t.Any
    nullable: t.Optional[bool]
    discriminator: t.Optional[Discriminator]
    read_only: t.Optional[bool] = Field(
        None,
        alias="readOnly",
    )
    write_only: t.Optional[bool] = Field(
        None,
        alias="writeOnly",
    )
    xml: t.Optional[XML]
    external_docs: t.Optional[ExternalDocs] = Field(
        None,
        alias="externalDocs",
    )
    example: t.Any
    deprecated: t.Optional[str]


class Example(BaseModelForbid):
    summary: t.Optional[str]
    description: t.Optional[str]
    value: t.Any
    externalValue: t.Optional[pydantic.AnyUrl]


class Header(BaseModelForbid):
    description: t.Optional[str]
    required: t.Optional[bool]
    deprecated: t.Optional[bool]
    allow_empty_value: t.Optional[bool] = Field(
        None,
        alias="allowEmptyValue",
    )
    schema_: t.Optional[SchemaUnion] = Field(
        None,
        alias="schema",
    )


HeadersUnion = t.Optional[t.Mapping[str, Header]]


class Encoding(BaseModelForbid):
    content_type: t.Optional[t.Union[str, MediaType]] = Field(
        None,
        alias="contentType",
    )
    headers: HeadersUnion
    style: t.Optional[str]
    explode: t.Optional[bool]
    allow_reserved: t.Optional[bool] = Field(
        None,
        alias="allowReserved",
    )


class MediaTypeObject(pydantic.BaseModel):
    schema_: t.Optional[SchemaUnion] = Field(
        None,
        alias="schema",
    )
    example: t.Any
    examples: t.Optional[t.Mapping[str, Example]]
    encoding: t.Optional[t.Mapping[str, Encoding]]


MediaTypeMap = t.Mapping[MediaType, MediaTypeObject]


class Response(BaseModelForbid):
    description: str
    content: t.Optional[MediaTypeMap]
    headers: HeadersUnion
    links: t.Optional[t.Mapping[str, "Link"]]


class RequestBody(BaseModelForbid):
    description: t.Optional[str]
    required: t.Optional[bool]
    content: t.Optional[MediaTypeMap]


class ServerVariables(BaseModelForbid):
    enum: t.Optional[t.List[str]]
    default: str
    description: t.Optional[str]


class Server(BaseModelAllow):
    url: t.Optional[str]
    description: t.Optional[str]
    variables: t.Optional[t.Mapping[str, ServerVariables]]


SecurityRequirement = t.Mapping[str, t.List[str]]


class Link(BaseModelForbid):
    operation_ref: t.Optional[str] = Field(
        None,
        alias="operationRef",
    )
    operation_id: t.Optional[str] = Field(
        None,
        alias="operationId",
    )
    request_body: t.Optional[t.Any] = Field(
        None,
        alias="requestBody",
    )
    parameters: t.Optional[t.Mapping[str, t.Any]]
    description: t.Optional[str]
    server: t.Optional[Server]


Response.update_forward_refs()


class Parameter(BaseModelForbid):
    # https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.2.md#parameterObject
    name: str
    in_: str = Field(
        alias="in",
    )
    description: t.Optional[str]
    required: t.Optional[bool]
    deprecated: t.Optional[bool]
    allow_empty_value: t.Optional[bool] = Field(
        None,
        alias="allowEmptyValue",
    )
    schema_: t.Optional[SchemaUnion] = Field(
        None,
        alias="schema",
    )
    style: t.Optional[str]
    explode: t.Optional[str]
    allow_reserved: t.Optional[bool] = Field(
        None,
        alias="allowReserved",
    )
    example: t.Any
    examples: t.Optional[Example]


Callback = t.Mapping[str, "PathItem"]


class Operation(BaseModelAllow):
    tags: t.Optional[t.List[str]]
    summary: t.Optional[str]
    description: t.Optional[str]
    external_docs: t.Optional[ExternalDocs] = Field(
        None,
        alias="externalDocs",
    )
    operation_id: t.Optional[str] = Field(
        None,
        alias="operationId",
    )
    responses: t.Mapping[HTTPStatusCode, Response]
    parameters: t.Optional[t.List[Parameter]]
    request_body: t.Optional[RequestBody] = Field(
        None,
        alias="requestBody",
    )
    callbacks: t.Optional[t.Mapping[str, Callback]]
    deprecated: t.Optional[bool]
    security: t.Optional[t.List[SecurityRequirement]]
    servers: t.Optional[t.List[Server]]


class PathItem(BaseModelAllow):
    summary: t.Optional[str]
    description: t.Optional[str]
    get: t.Optional[Operation]
    post: t.Optional[Operation]
    put: t.Optional[Operation]
    path: t.Optional[Operation]
    delete: t.Optional[Operation]
    head: t.Optional[Operation]
    options: t.Optional[Operation]
    trace: t.Optional[Operation]
    servers: t.Optional[t.List[Server]]
    parameters: t.Optional[t.Any]


Operation.update_forward_refs()


class OAuthFlowImplicit(BaseModelForbid):
    refresh_url: t.Optional[pydantic.AnyUrl] = Field(
        None,
        alias="refreshUrl",
    )
    scopes: t.Optional[t.Dict[str, str]]
    authorization_url: pydantic.AnyUrl = Field(alias="authorizationUrl")


class OAuthFlowPassword(BaseModelForbid):
    refresh_url: t.Optional[pydantic.AnyUrl] = Field(
        None,
        alias="refreshUrl",
    )
    scopes: t.Optional[t.Dict[str, str]]
    token_url: pydantic.AnyUrl = Field(alias="tokenUrl")


class OAuthFlowClientCredentials(BaseModelForbid):
    refresh_url: t.Optional[pydantic.AnyUrl] = Field(
        None,
        alias="refreshUrl",
    )
    scopes: t.Optional[t.Dict[str, str]]
    token_url: pydantic.AnyUrl = Field(alias="tokenUrl")


class OAuthFlowAuthorizationCode(BaseModelForbid):
    refresh_url: t.Optional[pydantic.AnyUrl] = Field(
        None,
        alias="refreshUrl",
    )
    scopes: t.Optional[t.Dict[str, str]]
    token_url: pydantic.AnyUrl = Field(alias="tokenUrl")
    authorization_url: pydantic.AnyUrl = Field(alias="authorizationUrl")


class OAuthFlows(BaseModelForbid):
    implicit: t.Optional[OAuthFlowImplicit]
    password: t.Optional[OAuthFlowPassword]
    client_credentials: t.Optional[OAuthFlowClientCredentials] = Field(
        None,
        alias="clientCredentials",
    )
    authorization_code: t.Optional[OAuthFlowAuthorizationCode] = Field(
        None,
        alias="authorizationCode",
    )


class SecuritySchemeOAuth2(BaseModelForbid):
    type: str = "oauth2"
    description: t.Optional[str]
    flows: OAuthFlows


class SecuritySchemeApiKey(BaseModelForbid):
    type: str = "apiKey"
    description: t.Optional[str]
    name: str
    in_: SecurityIn = Field(alias="in")


class SecuritySchemeOpenIdConnect(BaseModelForbid):
    type: str = "openIdConnect"
    description: t.Optional[str]
    open_id_connect_url: str = Field(alias="openIdConnectUrl")


class SecuritySchemeHttp(BaseModelForbid):
    type: str = "http"
    description: t.Optional[str]
    scheme: str
    bearer_format: t.Optional[str] = Field(
        None,
        alias="bearerFormat",
    )


SecuritySchemeUnion = t.Union[
    SecuritySchemeOAuth2,
    SecuritySchemeApiKey,
    SecuritySchemeOpenIdConnect,
    SecuritySchemeHttp,
]


class Components(BaseModelForbid):
    headers: HeadersUnion
    schemas: t.Optional[t.Mapping[str, Schema]]
    responses: t.Optional[t.Mapping[str, Response]]
    parameters: t.Optional[t.Mapping[str, Parameter]]
    examples: t.Optional[t.Mapping[str, Example]]
    request_bodies: t.Optional[t.Mapping[str, RequestBody]] = Field(
        None,
        alias="requestBodies",
    )
    links: t.Optional[t.Mapping[str, Link]]
    callbacks: t.Optional[t.Mapping[str, Callback]]
    security_schemes: t.Optional[t.Mapping[str, SecuritySchemeUnion]] = Field(
        None,
        alias="securitySchemes",
    )


Paths = t.Mapping[str, PathItem]


class Contact(BaseModelForbid):
    name: t.Optional[str]
    url: t.Optional[pydantic.AnyUrl]
    email: t.Optional[pydantic.EmailStr]


class License(BaseModelForbid):
    name: str
    url: t.Optional[pydantic.AnyUrl]


class Info(BaseModelAllow):
    contact: t.Optional[Contact]
    description: t.Optional[str]
    license: t.Optional[License]
    terms_of_service: t.Optional[pydantic.AnyUrl] = Field(
        None,
        alias="termsOfService",
    )
    title: str
    version: str


class Tag(BaseModelForbid):
    name: str
    description: t.Optional[str]
    external_docs: t.Optional[ExternalDocs] = Field(
        None,
        alias="externalDocs",
    )


AllClass = t.Union[
    Contact,
    Encoding,
    Example,
    ExternalDocs,
    Header,
    Info,
    License,
    Link,
    MediaTypeObject,
    OAuthFlowAuthorizationCode,
    OAuthFlowClientCredentials,
    OAuthFlowImplicit,
    OAuthFlowPassword,
    OAuthFlows,
    Operation,
    Parameter,
    PathItem,
    RequestBody,
    Response,
    Schema,
    SecuritySchemeApiKey,
    SecuritySchemeHttp,
    SecuritySchemeOAuth2,
    SecuritySchemeOpenIdConnect,
    Server,
    ServerVariables,
    Tag,
    XML,
]



import typing as t

import yaml

from openapydantic import common as common
from openapydantic import openapi_302 as openapi_302

OpenApi302 = openapi_302.OpenApi302
OpenApi = OpenApi302  # when antoher version will be implemented this should be a union
load_api_302 = openapi_302.load_api
OpenApiVersion = common.OpenApiVersion


async def load_spec(
    file_path: str,
    mode: t.Optional[str] = None,
) -> t.Dict[t.Any, t.Any]:
    if not mode:
        mode = "r"

    with open(file_path, "r") as file:
        result = yaml.safe_load(file)

    return result


async def load_api(
    *,
    file_path: str,
    version: t.Optional[OpenApiVersion] = None,
) -> OpenApi:
    raw_api = await load_spec(file_path=file_path)
    if not raw_api:
        raise ValueError("Api specification looks empty")

    spec_version = raw_api.get("openapi")

    if not spec_version:
        raise ValueError("openapi version not specified")

    if version == OpenApiVersion.v3_0_2 or spec_version == OpenApi302.__version__.value:
        return load_api_302(raw_api=raw_api)

    raise NotImplementedError(f"Unsupported openapi version:{spec_version}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['openapydantic', 'openapydantic.openapi_302']

package_data = \
{'': ['*']}

install_requires = \
['email-validator>=1.1.3,<2.0.0',
 'pydantic>=1.9.0,<2.0.0',
 'pyyaml>=5.3.1',
 'types-PyYAML>=6.0.5,<7.0.0']

setup_kwargs = {
    'name': 'openapydantic',
    'version': '0.2.0',
    'description': 'openapi specification parser based on pydantic',
    'long_description': '# Openapydantic\n\n[openapi](https://github.com/OAI/OpenAPI-Specification) specification validator based on [pydantic](https://pydantic-docs.helpmanual.io/).\n\n## Python version support\n\n\\>=3.8\n\n## Openapi versions support\n\n- ❌ 2.0\n- 🟠 3.0.0\n- 🟠 3.0.1\n- ✅ 3.0.2\n- ❌ 3.0.3\n- ❌ 3.1.0\n\nOpenapi versions are retrocompatibles (except for major version).\n\nSo 3.0.2 specification should be able to handle 3.0.0 and 3.0.1 data.\n\nUnit tests handle this case (3.0.2 object automatically try to load previous version fixtures).\n\n## Installation\n\nDepending on your preference...\n\n```\n    pip install openapydantic\n```\n\n...or...\n\n```\n    poetry add openapydantic\n```\n\n## Basic usage\n\n### Api loader\n\nOpenapydantic provide an openapi specification (a.k.a "swagger file" in version 2.X) loader.\n\nThis loader returns a pydantic model so you can work with your specification like a common pydantic python object.\n\nFor each openapi specification version, a dedicated python class exist.\n\nThe loader can either automatically determine the class to provide...\n\n```python\nimport asyncio\n\nimport openapydantic\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="openapi-spec.yaml",\n    ),\n)\nprint(api.info)\n# if my openapi version is "3.0.2", \'api\' is an instance of OpenApi302\n# if the version is not implemented, it will crash\n```\n\n... or you can also specify a specific version.\n\nIt may be useful for backward compatibility (for eg: create an OpenApi302 object using data from an 3.0.1 openapi specfication ).\n\n```python\nimport asyncio\n\nimport openapydantic\n\nOpenApiVersion = openapydantic.OpenApiVersion\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="openapi-spec-3-0-1.yaml",\n        version=OpenApiVersion.v3_0_2\n    ),\n)\n# Here ,\'api\' is an OpenApi302 object, event if you send an 3.0.1 spec.\n\nprint(api.openapi)\n>> 3.0.1 # version in the spec file\nprint(api.__version__)\n>> 3.0.2 # openapi version supported for the object class\n```\n\n### Reference interpolation\n\nOpenapydantic will interpolate openapi references.\n\nIf your api looks like this:\n\n```yaml\n# my-api.yaml\nopenapi: 3.0.2\ninfo:\n  version: "1.0.0"\n  title: Example\npaths:\n  /user:\n    get:\n      summary: Get user\n      responses:\n        "200":\n          description: successful operation\n          content:\n            application/json:\n              schema:\n                $ref: "#/components/schemas/User"\ncomponents:\n  schemas:\n    User:\n      type: object\n      properties:\n        id:\n          type: integer\n          format: int64\n        name:\n          type: string\n          example: "John Doe"\n```\n\nOnce loaded, it will be usable like if it was ...\n\n```yaml\nopenapi: 3.0.2\ninfo:\n  version: "1.0.0"\n  title: Example\npaths:\n  /user:\n    get:\n      summary: Get user\n      responses:\n        "200":\n          description: successful operation\n          content:\n            application/json:\n              schema:\n                type: object\n                properties:\n                  id:\n                    type: integer\n                    format: int64\n                  name:\n                    type: string\n                    example: "John Doe"\n\n```\n\nAnd so you will be able to do things like...\n\n```python\nimport asyncio\n\nimport openapydantic\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="my-api.yaml",\n    ),\n)\nprint(api.info)\nprint(\n    api.paths["/user"]\n    .get.responses["200"]\n    .content["application/json"]\n    .schema_.properties["name"]\n    .example\n)\n>> John Doe\n```\n\nAs describe in the openapi specification some attributes are fix (\'paths\', \'content\' etc...) and some can be mapping with a free key.\n\nMapping must be accessed like common dict, either by direct key loading, either using .get(\'*key*\')\n\nNote that file reference (e.g: "#/file.yaml" are currently not supported)\n\n### Attributes name collision\n\nOpenapi specify some attribute which name are already reserved either by pydantic,either by the python language itself.\n\nTo access these attributes, you must use the Openapydantic specific name\n\n| Attribute name | Openapydantic specific name |\n|----------------|------------------------------|\n| schema         | schema_                      |\n| in             | in_                          |\n| not            | not_                         |\n\ne.g:\n\n```python\nprint(\n    api.paths["/user"]\n    .get.responses["200"]\n    .content["application/json"]\n    .schema_\n)\n```\n\n### Model export\n\nYou can access the original api you provided as a dict using the **raw_api** attribute.\n\n```python\nimport asyncio\n\nimport openapydantic\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="my-api.yaml",\n    ),\n)\n\nprint(api.raw_api)\n>> {\'openapi\': \'3.0.2\', \'info\': {\'version\': \'1.0.0\', \'title\': \'Example\'}, \'paths\': {\'/user\': {\'get\': {\'summary\': \'Get user\', \'responses\': {\'200\': {\'description\': \'successful operation\', \'content\': {\'application/json\': {\'schema\': {\'$ref\': \'#/components/schemas/User\'}}}}}}}}, \'components\': {\'schemas\': {\'User\': {\'type\': \'object\', \'properties\': {\'id\': {\'type\': \'integer\', \'format\': \'int64\'}, \'name\': {\'type\': \'string\', \'example\': \'John Doe\'}}}}}}\n```\n\nYou can export your data as json string or as python dict using specific methods:\n\n```python\nimport asyncio\n\nimport openapydantic\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="my-api.yaml",\n    ),\n)\n\nprint(api.as_clean_json())\n>> {"openapi": "3.0.2", "info": {"title": "Example", "version": "1.0.0"}, "paths": {"/user": {"get": {"summary": "Get user", "responses": {"200": {"description": "successful operation", "content": {"application/json": {"schema": {"type": "object", "properties": {"id": {"type": "integer", "format": "int64"}, "name": {"type": "string", "example": "John Doe"}}}}}}}}}}}\n\n\nprint(api.as_clean_dict())\n> {\'openapi\': <OpenApiVersion.v3_0_2: \'3.0.2\'>, \'info\': {\'title\': \'Example\', \'version\': \'1.0.0\'}, \'paths\': {\'/user\': {\'get\': {\'summary\': \'Get user\', \'responses\': {\'200\': {\'description\': \'successful operation\', \'content\': {\'application/json\': {\'schema\': {\'type\': <JsonType.object_: \'object\'>, \'properties\': {\'id\': {\'type\': <JsonType.integer: \'integer\'>, \'format\': \'int64\'}, \'name\': {\'type\': <JsonType.string: \'string\'>, \'example\': \'John Doe\'}}}}}}}}}}}\n```\n\nNote that these functions are just wrapper to .dict() and .json() pydantic model with specific parameters.\n\nBy default, since the references are interpolated, the **components** root key is exclude.\n\nIf you want to have it in the output, you can set the **exclude_components** parameter to False.\n\n```python\nimport asyncio\n\nimport openapydantic\n\napi = asyncio.run(\n    openapydantic.load_api(\n        file_path="my-api.yaml",\n    ),\n)\n\nprint(\n    api.as_clean_json(\n        exclude_components=False,\n    ),\n)\n\n>> {"components": {"schemas": {"User": {"type": "object", "properties": {"id": {"type": "integer", "format": "int64"}, "name": {"type": "string", "example": "John Doe"}}}}}, "openapi": "3.0.2", "info": {"title": "Example", "version": "1.0.0"}, "paths": {"/user": {"get": {"summary": "Get user", "responses": {"200": {"description": "successful operation", "content": {"application/json": {"schema": {"type": "object", "properties": {"id": {"type": "integer", "format": "int64"}, "name": {"type": "string", "example": "John Doe"}}}}}}}}}}, "raw_api": {"openapi": "3.0.2", "info": {"version": "1.0.0", "title": "Example"}, "paths": {"/user": {"get": {"summary": "Get user", "responses": {"200": {"description": "successful operation", "content": {"application/json": {"schema": {"$ref": "#/components/schemas/User"}}}}}}}}, "components": {"schemas": {"User": {"type": "object", "properties": {"id": {"type": "integer", "format": "int64"}, "name": {"type": "string", "example": "John Doe"}}}}}}}\n\n```\n\nIn the same way,the **raw_api** attribute is exclude by default.\n\nIf you want to have it in the output, you can set the **exclude_raw_api** parameter to False.\n',
    'author': 'Richard Devers',
    'author_email': 'ritchiedev25@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/richarddevers/openapydantic',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.0.2.md

import typing as t

import pydantic

from openapydantic import common
from openapydantic.openapi_302 import models

Field = pydantic.Field

OpenApiVersion = common.OpenApiVersion


class OpenApi302(pydantic.BaseModel):
    __version__: t.ClassVar[OpenApiVersion] = OpenApiVersion.v3_0_2
    components: t.Optional[models.Components]
    openapi: OpenApiVersion
    info: models.Info
    paths: models.Paths
    tags: t.Optional[t.List[models.Tag]]
    servers: t.Optional[t.List[models.Server]]
    security: t.Optional[t.List[models.SecurityRequirement]]
    external_docs: t.Optional[models.ExternalDocs] = Field(
        None,
        alias="externalDocs",
    )
    raw_api: t.Dict[str, t.Any]

    class Config:
        extra = "forbid"

    def as_clean_json(
        self,
        *,
        exclude_components: bool = True,
        exclude_raw_api: bool = True,
    ) -> str:
        exclude: t.Set[str] = set()

        if exclude_components:
            exclude.add("components")

        if exclude_raw_api:
            exclude.add("raw_api")

        return self.json(
            by_alias=True,
            exclude_unset=True,
            exclude_none=True,
            exclude=exclude,
        )

    def as_clean_dict(
        self,
        *,
        exclude_components: bool = True,
        exclude_raw_api: bool = True,
    ) -> t.Dict[str, t.Any]:
        exclude: t.Set[str] = set()

        if exclude_components:
            exclude.add("components")

        if exclude_raw_api:
            exclude.add("raw_api")

        return self.dict(
            by_alias=True,
            exclude_unset=True,
            exclude_none=True,
            exclude=exclude,
        )


def load_api(
    *,
    raw_api: t.Dict[str, t.Any],
    clean_memory: bool = True,
) -> OpenApi302:
    models.ComponentsResolver.resolve(raw_api=raw_api)

    data: t.Dict[str, t.Any] = {
        **raw_api,
        "raw_api": raw_api,
    }

    api = OpenApi302(**data)
    if clean_memory:
        models.ComponentsResolver.init()
    return api



import enum
import typing as t

import pydantic


class ComponentType(enum.Enum):
    schemas = "schemas"
    headers = "headers"
    responses = "responses"
    parameters = "parameters"
    examples = "examples"
    request_bodies = "requestBodies"
    links = "links"
    callbacks = "callbacks"


class OpenApiBaseModel(pydantic.BaseModel):
    def as_clean_json(
        self,
        *,
        exclude_components: bool = True,
        exclude_raw_api: bool = True,
    ) -> str:
        exclude: t.Set[str] = set()

        if exclude_components:
            exclude.add("components")

        if exclude_raw_api:
            exclude.add("raw_api")

        return self.json(
            by_alias=True,
            exclude_unset=True,
            exclude_none=True,
            exclude=exclude,
        )

    def as_clean_dict(
        self,
        *,
        exclude_components: bool = True,
        exclude_raw_api: bool = True,
    ) -> t.Dict[str, t.Any]:
        exclude: t.Set[str] = set()

        if exclude_components:
            exclude.add("components")

        if exclude_raw_api:
            exclude.add("raw_api")

        return self.dict(
            by_alias=True,
            exclude_unset=True,
            exclude_none=True,
            exclude=exclude,
        )


# could be:
# -------
# import http
# HTTPStatusCode = t.Literal[tuple(str(x.value) for x in http.HTTPStatus)]
# HTTPStatusCode.append("default")
# -------
# but typing won't works with that so \o/
HTTPStatusCode = t.Literal[
    "default",  # openapi spec add
    "100",
    "101",
    "102",
    "200",
    "201",
    "202",
    "203",
    "204",
    "205",
    "206",
    "207",
    "208",
    "226",
    "300",
    "301",
    "302",
    "303",
    "304",
    "305",
    "307",
    "308",
    "400",
    "401",
    "402",
    "403",
    "404",
    "405",
    "406",
    "407",
    "408",
    "409",
    "410",
    "411",
    "412",
    "413",
    "414",
    "415",
    "416",
    "417",
    "421",
    "422",
    "423",
    "424",
    "426",
    "428",
    "429",
    "431",
    "451",
    "500",
    "501",
    "502",
    "503",
    "504",
    "505",
    "506",
    "507",
    "508",
    "510",
    "511",
]


HTTPMethod = t.Literal[
    "get",
    "put",
    "post",
    "patch",
    "delete",
    "head",
    "options",
]

MediaType = t.Literal[
    "application/json",
    "application/xml",
    "application/octet-stream",
    "application/x-www-form-urlencoded",
    "application/json; charset=utf-8",
]


class OpenApiVersion(enum.Enum):
    v3_0_0 = "3.0.0"
    v3_0_1 = "3.0.1"
    v3_0_2 = "3.0.2"
    # v3_0_3 = "3.0.3"
    # v3_1_0 = "3.1.0"
